
#include "LinkedCellList.h"

#include <stdio.h>
#include <string.h>


#include "atomarray.h"


//#define TEST1

LCL::LCL(double cellwidth) : 
	m_cell_count(0),
	m_cell_x(0),
	m_cell_y(0),
	m_cell_z(0),
	m_cell_bufsz(0),
	m_bufsz_atom(0),
	m_inext(NULL),
	m_icell(NULL),
	m_first_in_cell(NULL),
	m_auto_cellwidth(cellwidth),	//Z̏l
	m_orthogonality(true)		//WȂtrue
{

	
}


LCL::~LCL(){


	delete [] m_inext;
	delete [] m_icell;
	delete [] m_first_in_cell;

//	delete [] m_num_in_cell;
}

void LCL::SetMaxNumParticles(int maxcount){
	
	if(maxcount <= m_bufsz_atom){ return;}

	m_bufsz_atom = maxcount;

	if(m_inext) delete [] m_inext;
	m_inext = new int[maxcount];

	if(m_icell) delete [] m_icell;
	m_icell = new int[maxcount];

}

void LCL::SetBoxsizeCell(double boxsize_x, double boxsize_y, double boxsize_z, int cellcount_x, int cellcount_y, int cellcount_z){
	
	m_orthogonality = true;	//WȂtrue
	m_box.Set(boxsize_x, boxsize_y, boxsize_z);
	m_boxi.Set(1.0/boxsize_x, 1.0/boxsize_y, 1.0/boxsize_z);				//setting this only

#ifdef _DEBUG
	printf("set box: %lf, %lf, %lf\n", boxsize_x, boxsize_y, boxsize_z);
#endif

	m_Am.Set(m_box.x, 0.0, 0.0, 0.0, m_box.y, 0.0, 0.0, 0.0, m_box.z);

#ifdef TEST1	
	ResetCell(cellcount_x+2,cellcount_y+2,cellcount_z+2);
#else
	ResetCell(cellcount_x, cellcount_y, cellcount_z);
#endif

	RefreshPeriodicVector();

}


void LCL::EstimateCellCount(double boxsize_x, double boxsize_y, double boxsize_z, int* cellx, int* celly, int* cellz){

	
	int lx = (int)floor(boxsize_x/m_auto_cellwidth);
	int ly = (int)floor(boxsize_y/m_auto_cellwidth);
	int lz = (int)floor(boxsize_z/m_auto_cellwidth);
	if(lx <= 0){ lx = 1;}
	if(ly <= 0){ ly = 1;}
	if(lz <= 0){ lz = 1;}
	
	*cellx = lx;
	*celly = ly;
	*cellz = lz;
}

void LCL::SetBoxsize(double boxsize_x, double boxsize_y, double boxsize_z){

	
	int lx, ly, lz;
	EstimateCellCount(boxsize_x, boxsize_y, boxsize_z, &lx, &ly, &lz);

	SetBoxsizeCell(boxsize_x, boxsize_y, boxsize_z, lx, ly, lz);
}


////͐Ȃǂɂ񒼌̕ӂɂsim.box̐ݒ
void LCL::SetBoxAxis(const mat33d& Am){

	m_orthogonality = false;	//WȂtrue

	m_Am = Am;

	vec3d v = Cross(Am.b, Am.c);
	double abc = Am.a * v;
	double abs_b_c = Abs(v);
	v = Cross(Am.c, Am.a);
	double abs_c_a = Abs(v);
	v = Cross(Am.a, Am.b);
	double abs_a_b = Abs(v);
	

	//local ---------
	int lx,ly,lz;
	lx = (int)floor(abc/(m_auto_cellwidth * abs_b_c));
	ly = (int)floor(abc/(m_auto_cellwidth * abs_c_a));
	lz = (int)floor(abc/(m_auto_cellwidth * abs_a_b));		
	
#ifdef TEST1	
	ResetCell(lx+2,ly+2,lz+2);
#else
	ResetCell(lx,ly,lz);
#endif
	RefreshPeriodicVector();


	m_box.x = Am.a.x;
	m_box.y = Am.b.y;
	m_box.z = Am.c.z;
	m_boxi.x = 1.0/Am.a.x;
	m_boxi.y = 1.0/Am.b.y;
	m_boxi.z = 1.0/Am.c.z;

}

void LCL::SetBoxAxis(const vec3d& a, const vec3d& b, const vec3d& c){
		
		mat33d Am;
		Am.a = a;
		Am.b = b;
		Am.c = c;

		SetBoxAxis(Am);
}
/*
void LCL::SetPeriodicFlag(int periodic_flag){

	m_periodic_flag = periodic_flag;

}
*/



void LCL::ResetCell(int a_mcx, int a_mcy, int a_mcz){
//{bNXTCY`FbN, m_first_in_cellĊmۂ.
//mۂȂꍇ0Ԃ.
//obt@͊mۂcelľύXꍇ2Ԃ.


	//cell0̎҂͋֎~
	if(a_mcx <= 0){ a_mcx = 1;}
	if(a_mcy <= 0){ a_mcy = 1;}
	if(a_mcz <= 0){ a_mcz = 1;}

	//ZXV邩̃`FbN(trueȂXVȂ)
	if(m_cell_x == a_mcx){
		if(m_cell_y == a_mcy){
			if(m_cell_z == a_mcz){
				return;
			}
		}
	}
	
	
	m_cell_x = a_mcx;
	m_cell_y = a_mcy;
	m_cell_z = a_mcz;
	m_cell_count = m_cell_x * m_cell_y * m_cell_z;


//cellɎ̍Ċm=============================
	
	int realloc_flag = 0;

	if((m_cell_bufsz < m_cell_count) || 
	   (m_cell_bufsz > m_cell_count * 2)) {	//bufferĊm.

		m_cell_bufsz = m_cell_count * 2;	//obt@TCYɗ]T.
		
		
	//	#pragma omp master
	//	printf("allocate %d, %d, %d)\n",a_mcx,a_mcy,a_mcz);
		delete [] m_first_in_cell;
		m_first_in_cell = new int[m_cell_bufsz +1];		//master threadŃP[g.

//		delete [] m_num_in_cell;
//		m_num_in_cell = new int[m_cell_bufsz];		//master threadŃP[g.


		realloc_flag = 1;

		
	}

}



void LCL::RefreshPeriodicVector(){

	//EׂɗqԂ̑΃xNg獷xNg//
	if(m_orthogonality){	//n
		int index = 0;
		for(int iz=0; iz < 3; iz++){
			for(int iy=0; iy < 3; iy++){
				for(int ix=0; ix < 3; ix++){
					switch(ix){
						case 0: m_periodic_vec[index].x = 0.0; break;
						case 1: m_periodic_vec[index].x = m_box.x; break;
						case 2: m_periodic_vec[index].x =-m_box.x; break;
					}
					switch(iy){
						case 0: m_periodic_vec[index].y = 0.0; break;
						case 1: m_periodic_vec[index].y = m_box.y; break;
						case 2: m_periodic_vec[index].y =-m_box.y; break;
					}
					switch(iz){
						case 0: m_periodic_vec[index].z = 0.0; break;
						case 1: m_periodic_vec[index].z = m_box.z; break;
						case 2: m_periodic_vec[index].z =-m_box.z; break;
					}
					index++;
				}
			}
		}
	}else{				//񒼌n
		int index = 0;
		vec3d zero = {0.0,0.0,0.0};
		for(int iz=0; iz < 3; iz++){
			for(int iy=0; iy < 3; iy++){
				for(int ix=0; ix < 3; ix++){
					switch(ix){
						case 0: m_periodic_vec[index] = zero; break;
						case 1: m_periodic_vec[index] = m_Am.a; break;
						case 2: m_periodic_vec[index] =-m_Am.a; break;
					}
					switch(iy){
						//case 0: m_periodic_vec[index].y = 0.0; break;
						case 1: m_periodic_vec[index] += m_Am.b; break;
						case 2: m_periodic_vec[index] -= m_Am.b; break;
					}
					switch(iz){
						//case 0: m_periodic_vec[index].z = 0.0; break;
						case 1: m_periodic_vec[index] += m_Am.c; break;
						case 2: m_periodic_vec[index] -= m_Am.c; break;
					}
					index++;
				}
			}
		}
	
	}


}


//



void LCL::Create(int count, vec3d* __restrict r, const int* __restrict knd){
	
	Clear();

	Calculate(0, count, r, knd);

	
}

void LCL::Create(int istart, int count, vec3d* __restrict r, const int* __restrict knd){
	
	Clear();

	Calculate(istart, count, r, knd);

	
}




void LCL::Calculate(int istart, int count, vec3d* __restrict r, const int* __restrict knd){


	int iend = istart + count;
	int* __restrict re_icell = m_icell;
		
	if(m_orthogonality){
//n̏ꍇ////////////////////////////////////////////////////////
		
		
		const double boxix = m_boxi.x;
		const double boxiy = m_boxi.y;
		const double boxiz = m_boxi.z;

		
		const int cell_x = m_cell_x;
		const int cell_y = m_cell_y;
		//const int cell_z = m_cell_z;
		const double d_cell_x = (double)m_cell_x;
		const double d_cell_y = (double)m_cell_y;
		const double d_cell_z = (double)m_cell_z;

		
		#pragma omp parallel for
		for(int i = istart; i < iend ; i++){		

			double cx,cy,cz;
			
			//double xi = r[i].x * m_boxi.x;
			double xi = r[i].x * boxix;
			//xi -= floor(xi + 1.0) - 1.0;
			xi = modf(xi + 1.0, &cx);
			//xi = modf(xi+1.0, NULL);
	//		m_cycle_x[i] += (int)cx;
	//		m_cycle_x[i] --;
			r[i].x = xi * m_box.x;
			
			//double yi = r[i].y * m_boxi.y;
			double yi = r[i].y * boxiy;
			//yi -= floor(yi + 1.0) - 1.0;
			yi = modf(yi+1.0, &cy);
	//		m_cycle_y[i] += (int)cy;
	//		m_cycle_y[i] --;
			r[i].y = yi * m_box.y;
			
			//double zi = r[i].z * m_boxi.z;
			double zi = r[i].z * boxiz;
			//zi -= floor(zi + 1.0) - 1.0;
			zi = modf(zi+1.0, &cz);
	//		m_cycle_z[i] += (int)cz;
	//		m_cycle_z[i] --;
			r[i].z = zi * m_box.z;

			//	m_icell[i] = (int)floor(xi * (double)(m_cell_x)) + ( (int)floor(yi * (double)(m_cell_y)) + (int)floor(zi * (double)(m_cell_z))*m_cell_y )*m_cell_x;
			re_icell[i] = ((int)floor(xi * d_cell_x)) + ((int)floor(yi * d_cell_y) + ((int)floor(zi * d_cell_z))*cell_y )*cell_x;
		}
		
	/*	
		const double boxx = m_box.x;
		const double boxy = m_box.y;
		const double boxz = m_box.z;

		const double boxix = m_boxi.x;
		const double boxiy = m_boxi.y;
		const double boxiz = m_boxi.z;

		const int cell_x = m_cell_x;
		const int cell_y = m_cell_y;
		//const int cell_z = m_cell_z;
		const double d_cell_x = (double)m_cell_x;
		const double d_cell_y = (double)m_cell_y;
		const double d_cell_z = (double)m_cell_z;

		const int cell_xm1 = m_cell_x - 1;
		const int cell_ym1 = m_cell_y - 1;
		const int cell_zm1 = m_cell_z - 1;
		
		#pragma omp parallel for
		for(int i = istart; i < iend ; i++){		

			double cx,cy,cz;
			
			
			double xi = r[i].x * boxix;
			xi = modf(xi, &cx);
			int idx;
			if(xi < 0.0){
				idx = (int)(xi * d_cell_x) + cell_xm1;
				xi += 1.0;
			}else{
				idx = (int)(xi * d_cell_x);
			}
			r[i].x = xi * boxx;
			
			double yi = r[i].y * boxiy;
			yi = modf(yi, &cy);
			int idy;
			if(yi < 0.0){
				idy = (int)(yi * d_cell_y) + cell_ym1;
				yi += 1.0;
			}else{
				idy = (int)(yi * d_cell_y);
			}
			r[i].y = yi * boxy;
			
			double zi = r[i].z * boxiz;
			zi = modf(zi, &cz);
			int idz;
			if(zi < 0.0){
				idz = (int)(zi * d_cell_z) + cell_zm1;
				zi += 1.0;
			}else{
				idz = (int)(zi * d_cell_z);
			}
			r[i].z = zi * boxz;

			re_icell[i] = idx + ( idy + idz*cell_y )*cell_x;

		}
	*/
	/*
		
		const double boxx = m_box.x;
		const double boxy = m_box.y;
		const double boxz = m_box.z;

		const double boxix = m_boxi.x;
		const double boxiy = m_boxi.y;
		const double boxiz = m_boxi.z;

		const int cell_x = m_cell_x;
		const int cell_y = m_cell_y;
		const double d_cell_x = ((double)m_cell_x)/boxx;
		const double d_cell_y = ((double)m_cell_y)/boxy;
		const double d_cell_z = ((double)m_cell_z)/boxz;

		#pragma omp parallel for
		for(int i = istart; i < iend ; i++){		

			
			//double xi = fmod(r[i].x, boxx);
			//r[i].x = xi;
			double xi = r[i].x;
			if (xi < 0.0){ xi += boxx;}
			else if (xi >= boxx){ xi -= boxx;}
			r[i].x = xi;

			
			
			double yi = r[i].y;
			if (yi < 0.0){ yi += boxy;}
			else if (yi >= boxy){ yi -= boxy;}
			r[i].y = yi;
			
			double zi = r[i].z;
			if (zi < 0.0){ zi += boxz;}
			else if (zi >= boxz){ zi -= boxz;}
			r[i].z = zi;

		//	m_icell[i] = (int)floor(xi * (double)(m_cell_x)) + ( (int)floor(yi * (double)(m_cell_y)) + (int)floor(zi * (double)(m_cell_z))*m_cell_y )*m_cell_x;
		//	re_icell[i] = (int)floor(xi * d_cell_x) + ( (int)floor(yi * d_cell_y) + (int)floor(zi * d_cell_z)*cell_y )*cell_x;
			re_icell[i] = (int)(xi * d_cell_x) + ( (int)(yi * d_cell_y) + (int)(zi * d_cell_z)*cell_y )*cell_x;
			
			//if(re_icell[i] < 0 || re_icell[i] >= m_cell_count){
			//	re_icell[i] = re_icell[i];
			//}
			

		}
		*/
	}else{
//񒼌n̏ꍇ////////////////////////////////////////////////////////
		
		/*
		const mat33d Am = m_Am;
		const mat33d iAm = Inverse(m_Am);

		const int cell_x = m_cell_x;
		const int cell_y = m_cell_y;
		const double d_cell_x = (double)m_cell_x;
		const double d_cell_y = (double)m_cell_y;
		const double d_cell_z = (double)m_cell_z;

		#pragma omp parallel for
		for(int i = istart; i < iend ; i++){		
	
//			if (knd[i] < -1) continue;
			
			double cx,cy,cz;
			
			vec3d r_prim = iAm * r[i];

			r_prim.x = modf(r_prim.x+1.0, &cx);
			//r_prim.x -= floor(r_prim.x + 1.0) - 1.0;
	//		m_cycle_x[i] += (int)cx;
	//		m_cycle_x[i] --;
			
			r_prim.y = modf(r_prim.y+1.0, &cy);
			//r_prim.y -= floor(r_prim.y + 1.0) - 1.0;
	//		m_cycle_y[i] += (int)cy;
	//		m_cycle_y[i] --;
			
			r_prim.z = modf(r_prim.z+1.0, &cz);
			//r_prim.z -= floor(r_prim.z + 1.0) - 1.0;
	//		m_cycle_z[i] += (int)cz;
	//		m_cycle_z[i] --;
			
			r[i] = Am * r_prim;

			//m_icell[i] = (int)(r_prim.x * (double)(m_cell_x)) + ( (int)(r_prim.y * (double)(m_cell_y)) + (int)(r_prim.z * (double)(m_cell_z))*m_cell_y )*m_cell_x;
			re_icell[i] = (int)(r_prim.x * d_cell_x) + ( (int)(r_prim.y * d_cell_y) + (int)(r_prim.z * d_cell_z)*cell_y )*cell_x;
			
		}
		*/

		
		const mat33d Am = m_Am;
		const mat33d iAm = Inverse(m_Am);

		const int cell_x = m_cell_x;
		const int cell_y = m_cell_y;
		const double d_cell_x = (double)m_cell_x;
		const double d_cell_y = (double)m_cell_y;
		const double d_cell_z = (double)m_cell_z;

		const int cell_xm1 = m_cell_x - 1;
		const int cell_ym1 = m_cell_y - 1;
		const int cell_zm1 = m_cell_z - 1;
		
		#pragma omp parallel for
		for(int i = istart; i < iend ; i++){		
	
//			if (knd[i] < -1) continue;
			
			double cx,cy,cz;
			
			vec3d r_prim = iAm * r[i];
			r_prim.x = modf(r_prim.x, &cx);
			int idx;
			if(r_prim.x < 0.0){
				idx = (int)(r_prim.x * d_cell_x) + cell_xm1;
				r_prim.x += 1.0;
			}else{
				idx = (int)(r_prim.x * d_cell_x);
			}

			r_prim.y = modf(r_prim.y, &cy);
			int idy;
			if(r_prim.y < 0.0){
				idy = (int)(r_prim.y * d_cell_y) + cell_ym1;
				r_prim.y += 1.0;
			}else{
				idy = (int)(r_prim.y * d_cell_y);
			}
			
			r_prim.z = modf(r_prim.z, &cz);
			int idz;
			if(r_prim.z < 0.0){
				idz = (int)(r_prim.z * d_cell_z) + cell_zm1;
				r_prim.z += 1.0;
			}else{
				idz = (int)(r_prim.z * d_cell_z);
			}

			r[i] = Am * r_prim;
			re_icell[i] = idx + ( idy + idz*cell_y )*cell_x;
			
		}

	}	//m_orthogonality//

	

	{
		for(int i = iend - 1; i >= istart; i--){
				
			//if (knd[i] < -1) continue;
			const int icell = re_icell[i];		
			

			m_inext[i] = m_first_in_cell[icell];
			m_first_in_cell[icell] = i;
			
		}
		
	}
	
}

void LCL::GetCellInfo(LCL_INFO* info, vec3d *periodic_vec){
	
	
//return infomation//
	info->cell_x = m_cell_x;
	info->cell_y = m_cell_y;
	info->cell_z = m_cell_z;
	info->icell = m_icell;
	info->first_in_cell = m_first_in_cell;
	info->inext = m_inext;
	memcpy(periodic_vec, m_periodic_vec, sizeof(vec3d)*27);

}


const vec3d* LCL::GetCellInfo(LCL_INFO* info){
	
	
//return infomation//
	info->cell_x = m_cell_x;
	info->cell_y = m_cell_y;
	info->cell_z = m_cell_z;
	info->icell = m_icell;
	info->first_in_cell = m_first_in_cell;
	info->inext = m_inext;
	return m_periodic_vec;

}
void LCL::GetCellWidth(vec3d *width){
	
	width->Set(m_box.x / (double)m_cell_x, m_box.y / (double)m_cell_y, m_box.z / (double)m_cell_z);
	
}

	




void LCL::Clear(){
	// set zero head of chain array
	//#pragma omp master
	{
		memset(m_first_in_cell, -1, m_cell_count * sizeof(int));
	}
}


#ifdef no_used
//void LCL::SortLinkedList(int count, vec3d* __restrict r, vec3d* __restrict rehash_r){
void LCL::SortLinkedList(int count, vec3d* __restrict r, vec3d* __restrict rehash_r, int* __restrict knd, int* __restrict rehash_knd){
	//łm_first_in_cell̃NA͂Ȃ̂ŁAOClearLinkedListĂԂ//

	int cell_count = m_cell_count;
	//int* __restrict re_icell = m_icell;
	int* __restrict first_in_cell = m_first_in_cell;
//	int* __restrict num_in_cell = m_num_in_cell;
	int* __restrict inext = m_inext;
	int* __restrict iindex = m_iindex;

	/*case 1
	#pragma omp for
	for(int i = 0; i < cell_count; i++){
		
		int atom_count = 0;
		for (int j = first_in_cell[i]; j != -1; j = inext[j]){
			atom_count++;
		}
						
		num_in_cell[i] = atom_count;
	}

	
	int th_count = omp_get_num_threads();
	int th_atom_count = count / th_count;
	for(int i = 0; i < cell_count; i++){

	 */
	
	#pragma omp single
	{
		int atom_index = 0;
		for(int i = 0; i < cell_count; i++){
		
			int j_start = first_in_cell[i];
			first_in_cell[i] = atom_index;

			int j_next;
			for (int j = j_start; j != -1; j = j_next){
				j_next = inext[j];
				//inext[j] = atom_index;
				iindex[j] = atom_index;
				atom_index++;
			}

			//num_in_cell[i] = atom_index;
		}
		first_in_cell[cell_count] = atom_index;
	}

	#pragma omp for
	for(int i = 0; i < count; i++){
	//	re_icell[inext[i]] = i;
		inext[iindex[i]] = i;
		rehash_r[iindex[i]] = r[i];
		rehash_knd[iindex[i]] = knd[i];
	}
	

}

void LCL::SortLinkedList2(int count, vec3d* __restrict r, double* __restrict rehash_x, double* __restrict rehash_y, double* __restrict rehash_z){
	//łm_first_in_cell̃NA͂Ȃ̂ŁAOClearLinkedListĂԂ//

	int cell_count = m_cell_count;
	//int* __restrict re_icell = m_icell;
	int* __restrict first_in_cell = m_first_in_cell;
//	int* __restrict num_in_cell = m_num_in_cell;
	int* __restrict inext = m_inext;
	int* __restrict iindex = m_iindex;


	#pragma omp single
	{
		int atom_index = 0;
		for(int i = 0; i < cell_count; i++){
		
			int j_start = first_in_cell[i];
			first_in_cell[i] = atom_index;

			int j_next;
			for (int j = j_start; j != -1; j = j_next){
				j_next = inext[j];
				//inext[j] = atom_index;
				iindex[j] = atom_index;
				atom_index++;
			}

			//num_in_cell[i] = atom_index;
		}
		first_in_cell[cell_count] = atom_index;
	}

	#pragma omp for
	for(int i = 0; i < count; i++){
	//	re_icell[inext[i]] = i;
		inext[iindex[i]] = i;
		rehash_x[iindex[i]] = r[i].x;
		rehash_y[iindex[i]] = r[i].y;
		rehash_z[iindex[i]] = r[i].z;
	}
	

}
#endif