#ifndef __LinkedCellList_h__
#define __LinkedCellList_h__

#pragma once


#include "vec3.h"
#include "mat33.h"


struct LCL_INFO{
	int cell_x;	//x̃Z//
	int cell_y;	//ỹZ//
	int cell_z;	//z̃Z//
	int* icell;	//q̓ĂZԍ//
	int* inext;	//cellł̗̎qւ̃N//
	int* first_in_cell;	//cellɓĂ擪̗qԍ//
};

struct BVEC_REGION{
	int startpoint;		// start index of i-th atom's bond in BOP_BVECTOR array //
	int endpoint;		// end index of i-th atom's bond in BOP_BVECTOR array //
	int twobodyendpoint;
};


class LCL
{
protected:
	//int m_max_count;	//肦ő嗱q

	vec3d m_box;						
	vec3d m_boxi;
	mat33d m_Am;		//sim.box̎e\(a, b, c).
	bool m_orthogonality;	//WȂtrue

	int m_bufsz_atom;
	int* m_inext;		//cellɓĂ鎟̗qԂ(qƂ̃f[^)//
	int* m_icell;		//q̓ĂZ̔ԍ(qƂ̃f[^)//
	//int* m_iindex;		

	
	int m_cell_count;	//Scell(m_cell_x * m_cell_y * m_cell_z)
	int m_cell_x;		//xcell
	int m_cell_y;		//ycell
	int m_cell_z;		//zcell
	
	int m_cell_bufsz;	//cellƂ̔z̃obt@TCY(Ċmێɂ𒴂realloc).
	
	int* m_first_in_cell;	//cellɓĂ擪̗qԍ(cellƂ̃f[^).//omp:shared//
//	int* m_num_in_cell;	//cellɓĂ擪̗qԍ(cellƂ̃f[^).//omp:shared//

	vec3d m_periodic_vec[27];

	double m_auto_cellwidth;

	

	void ResetCell(int a_mcx, int a_mcy, int a_mcz);

	void RefreshPeriodicVector();
	
	
public:
	
	LCL(double cellwidth);
	virtual ~LCL();
	
	void SetMaxNumParticles(int maxcount);
	void SetBoxsizeCell(double boxsize_x, double boxsize_y, double boxsize_z, int cellcount_x, int cellcount_y, int cellcount_z);
	void EstimateCellCount(double boxsize_x, double boxsize_y, double boxsize_z, int* cellx, int* celly, int* cellz);
	void SetBoxsize(double x, double y, double z);
	void SetBoxAxis(const vec3d& a, const vec3d& b, const vec3d& c);
	void SetBoxAxis(const mat33d& Am);
	//mat33d GetBoxAxis();

	//void SetPeriodicFlag(int periodic_flag);
	
	
	//void GetLinkedList(int count, vec3d* r, int* knd, LCL_INFO* info, vec3d *periodic_vec);
	void Create(int count, vec3d* __restrict r, const int* __restrict knd);
	void Create(int istart, int count, vec3d* __restrict r, const int* __restrict knd);
	void GetCellInfo(LCL_INFO* info, vec3d *periodic_vec);
	const vec3d* GetCellInfo(LCL_INFO* info);
	void GetCellWidth(vec3d *width);
	
	void Clear();
	void Calculate(int istart, int count, vec3d* __restrict r, const int* __restrict knd);
	
	template <class TBVECTOR>
	int CreateBondList(int start_index, int count, const vec3d* r, double CUTOFF_SQR, TBVECTOR* bvector, BVEC_REGION* bvecStartEnd, int buf_sz_bond);

#ifdef no_use
	//void SortLinkedList(int count, vec3d* __restrict r, vec3d* __restrict rehash_r);
	void SortLinkedList(int count, vec3d* __restrict r, vec3d* __restrict rehash_r, int* __restrict knd, int* __restrict rehash_knd);
	void SortLinkedList2(int count, vec3d* __restrict r, double* __restrict rehash_x, double* __restrict rehash_y, double* __restrict rehash_z);
#endif

};

/////////////////////////////////////////////
//Create Bond List by using result from LCL//
/////////////////////////////////////////////


#include "LCL_CreateBondList.h"

#if 0
/*
{hXg.
obt@ȂƂ-1Ԃ
*/
template <class TBVECTOR>
int CreateBondList(int start_index, int count, const vec3d* r, double CUTOFF_SQR, LCL* pLCL, TBVECTOR* bvector, BVEC_REGION* bvecStartEnd, int buf_sz_bond){
	// local variable ------------------------------;

	LCL_INFO info;
	vec3d periodic_vec[27];
	pLCL->GetCellInfo(&info, periodic_vec);
	
	const int cell_wx = info.cell_x;
	const int cell_wy = info.cell_y;
	const int cell_wz = info.cell_z;
	const int* icell = info.icell;
	const int* first_in_cell = info.first_in_cell;
	const int* inext = info.inext;
	
	const int cell_wxwy = cell_wx * cell_wy;
	const int cell_wxwywz = cell_wxwy * cell_wz;



#ifdef _OPENMP
	// shift array pointer for each thread //
	const int th = omp_get_thread_num();
	const int thread_count = omp_get_num_threads();
	int bond_index = th * (buf_sz_bond/thread_count) + 1;
#else
	const int th = 0;
	const int thread_count = 1;
	int bond_index = 1;
#endif


	//int max_num = 0;
	
	//openmp parallelization for atoms//

	const int end_index = start_index + count;
	
	#pragma omp for
	for(int i = start_index; i < end_index; i++){
		const vec3d r_i = r[i];
		const int ic = icell[i];
		//const int ki = m_knd[i];
		const int start_point = bond_index;


		const int icell_z = ic / cell_wxwy;
		int nz[3] = {0, 0, 0};
		int tz[3] = {-cell_wxwy, 0, cell_wxwy};
		
		if(icell_z == 0){
			nz[0] = 18;	//2 * (3 * 3)
			tz[0] += cell_wxwywz;
		}
		if (icell_z == cell_wz - 1){
			nz[2] = 9;	//1 * (3 * 3)
			tz[2] -= cell_wxwywz;
		}

		const int icell_y = (ic / cell_wx) % cell_wy;
		int ny[3] = {0, 0, 0};
		int ty[3] = {-cell_wx, 0, cell_wx};
		if(icell_y == 0){
			ny[0] = 6;	//2 * 3
			ty[0] += cell_wxwy;
		}
		if (icell_y == cell_wy - 1){
			ny[2] = 3;	//1 * 3
			ty[2] -= cell_wxwy;
		}

		const int icell_x = ic % cell_wx;
		int nx[3] = {0, 0, 0};
		int tx[3] = {-1, 0, 1};
		if(icell_x == 0){
			nx[0] = 2;
			tx[0] = (cell_wx - 1);
		}
		if (icell_x == cell_wx - 1){
			nx[2] = 1;
			tx[2] = (1 - cell_wx);
		}

		// loop for adjoin cell and same cell//
		//int num_t = 0;
		for(int iz = 0; iz < 3; iz++){
			for(int iy = 0; iy < 3; iy++){
				for(int ix= 0; ix < 3; ix++){

					const int nxyz = nx[ix] + ny[iy] + nz[iz];
					const int target_cell = ic + tx[ix] + ty[iy] + tz[iz];
					
					//if(target_cell == ic) continue;
					
					//if(nxyz){//قȂZ//(PperiodicEׂȂɂȂĂ܂Ă)
					//if(target_cell != ic){//قȂZ//(unit cell̍ۂɂׂĂZɂȂĂ܂)
					if(((ix & (iy & iz)) & 0x1) == 0){//(unit cellł)
					
					
						const vec3d rb_i = r_i - *(periodic_vec + nxyz);
						for (int j = first_in_cell[target_cell]; j != -1; j = inext[j]){
							const vec3d dd = rb_i - r[j];

							const double r2 = dd * dd;

							// judge the relative distance //
							if (r2 <= CUTOFF_SQR) {
		
								// add to bonding array //
								TBVECTOR& bv_i = bvector[bond_index];

								bv_i.pairIndex = j;
								bv_i.rij = dd;
								bv_i.dr = sqrt(r2);
		
								// increment of index //
								bond_index ++;
							}
						}
						
					
					}else{//Z//
						const vec3d rb_i = r_i;
						for (int j = first_in_cell[target_cell]; j != -1; j = inext[j]){
							if(i == j){
								//x܂ł13/27ZƓZ̑OI//
								bvecStartEnd[i].twobodyendpoint = bond_index;	//end point for two-body force of i-th particle's bond//
							}else{
								const vec3d dd = rb_i - r[j];
								const double r2 = dd * dd;

								// judge the relative distance //
								if (r2 <= CUTOFF_SQR) {
		
									// add to bonding array //
									TBVECTOR& bv_i = bvector[bond_index];

									bv_i.pairIndex = j;
									bv_i.rij = dd;
									bv_i.dr = sqrt(r2);
		
									// increment of index //
									bond_index ++;
								}
							}
						}
					}
		

				}
			}
		}//iz//


		bvecStartEnd[i].startpoint = start_point;	//start point of i-th particle's bond array//
		bvecStartEnd[i].endpoint = bond_index;	//end point of i-th particle's bond//

	}//end of for i//

	//check of bond buffer size//
	{
		const int limit_sz = (th+1) * (buf_sz_bond/thread_count) + 1;
		if(bond_index >= limit_sz){
			return -1;
		}
	}
	return 0;

};
#endif

#endif    // __LinkedCellList_h__
