
#include "RenderDX11.h"
#include "mod.h"
#include "visual.h"
#include "camera.h"

#include "InstanceRenderer.h"
//#include "PointSpriteRenderer.h"

#include "AtomRendererDX11.h"
#include "PointRendererDX11.h"

#include "MDLoader.h"
#include "filelist.h"

#include "setting.h"

#define _USE_MATH_DEFINES
#include <string.h>
#include <math.h>


#define WIN32_LEAN_AND_MEAN             // Windows wb_[gpĂȂO܂B
#include "windib2.h"
#include <windows.h>
#include <tchar.h>


//DirectX11 system//////////////////////////////
static DX11_CORE* s_dx11_core = NULL;
static CommonShader* comShader = NULL;
//static CommonTexture* comTexture = NULL;

//static InstanceRenderer* instRdr = NULL;
//static PointSpriteRenderer* psRdr = NULL;
static AtomRendererDX11* m_atomRendererDX11 = NULL;
static PointRendererDX11* m_pointRendererDX11 = NULL;

//polygon//////////////////////////////////////////////
static vec3f* flier_vertex;
static int flier_vertex_count;
static WORD* flier_index;
static int flier_index_count;

extern double VIEW_ANGLE;
extern double VIEW_NEAR;
extern double VIEW_FAR;



//extern MasterFileList g_files;		//Jt@C̃Xg.
extern bool g_updateFlag;	//_OȂ悤LockB]Ȃ.

///////////////////////////////////////////////////////////////////
//ϊspCu
///////////////////////////////////////////////////////////////////

static void SetViewMatrix(vec3f* target, vec3f* origin, vec3f* up, mat44f* pViewMatrix);
static void SetProjectionPerspectiveMatrix(float angle, float wph, float z_near, float z_far, mat44f* pProjectionMatrix);



///////////////////////////////////////////////////////////////////
//ȉ̓[U[`̃R[obN֐
///////////////////////////////////////////////////////////////////

void InitDX11(DX11_CORE* dx11_core){
	
	s_dx11_core = dx11_core;
	
	//ShaderpǗNX
	comShader = new CommonShader(dx11_core->pd3dDevice);
	
	//q|S.
	CreatePolygonSphere(0.5f, g_visual_atompoly, g_visual_atompoly*2, &flier_vertex, &flier_vertex_count, &flier_index, &flier_index_count);


}

void TerminateDX11(){

	if(m_pointRendererDX11){ delete m_pointRendererDX11;}
	if(m_atomRendererDX11){ delete m_atomRendererDX11;}
	
	if(comShader){ delete comShader;}

}

void RenderingDX11(VP_FileList& files, int screen_w, int screen_h, CameraGL* cam){

	DX11_CORE* dx11_core = s_dx11_core;
	
	//`̃Zbg///////////////////////////////////////////
	//BackBuffer̃NA
	dx11_core->pd3dDC->ClearRenderTargetView( dx11_core->pRenderTargetView, g_bg_color);
	    
	//DepthBuffer̃NA
	dx11_core->pd3dDC->ClearDepthStencilView( dx11_core->pDepthStencilView, D3D11_CLEAR_DEPTH, 1.0f, 0);
	
	//ϊsvZ
	vec3f target, camera, up;
	cam->GetInfo(&target, &camera, &up);
	mat44f view_matrix;
	SetViewMatrix(&target, &camera, &up, &view_matrix); 
		
	mat44f projection_matrix;
	SetProjectionPerspectiveMatrix((float)(20.0f/180.0f*M_PI), (float)screen_w/(float)screen_h, 10.0f, 10000.0f, &projection_matrix);
		
	
	//visual setting//
	VISUAL_SETTING vis;
	vis.atom = g_visual_atom;
	vis.atom_poly = g_visual_atompoly;
	vis.atom_radius = g_visual_atomradius;
	vis.atom_color = g_visual_atomcolor;
	vis.bond = g_visual_bond;
	vis.bond_poly = g_visual_bondpoly;

	//et@CƂ̃_O//
	
	for(ITP_FileList it = files.begin(); it != files.end(); it++){
		FileInfo* fi = *it;
	
	
		switch(fi->type){
			case FILETYPE_ATOM:
			{
				if (vis.atom == VISUAL_ATOM_POINT){

			
					//_O///////////////////////////////////////////
			
					if(m_pointRendererDX11 == NULL) m_pointRendererDX11 = new PointRendererDX11(dx11_core, comShader);
			
					m_pointRendererDX11->SetViewMatrixPointer( &view_matrix );
					m_pointRendererDX11->SetProjectionMatrixPointer( &projection_matrix );
					ATOMS_DATA* dat = fi->atm->GetDataPointer();

					m_pointRendererDX11->Draw(dat, &(fi->bond), vis);
			

				}else if(g_visual_atom == VISUAL_ATOM_SPHERE){


					//_O///////////////////////////////////////////
			
					if(m_atomRendererDX11 == NULL) m_atomRendererDX11 = new AtomRendererDX11(dx11_core, comShader);
			
					m_atomRendererDX11->SetViewMatrixPointer( &view_matrix );
					m_atomRendererDX11->SetProjectionMatrixPointer( &projection_matrix );
					ATOMS_DATA* dat = fi->atm->GetDataPointer();

					m_atomRendererDX11->Draw(dat, &(fi->bond), vis);
			

				}
			}
			break;
			
		case FILETYPE_FIELD:
			{
			}
			break;
		}		

	}//loop

}


///////////////////////////////////////////////////////////////////
//ϊspCu
///////////////////////////////////////////////////////////////////

void SetViewMatrix(vec3f* target, vec3f* origin, vec3f* up, mat44f* pViewMatrix){
	
	/////////////////////////////////////////////////////////////////////
	//WnorEWn͂Ō܂//
	//Wn : ʉEx,ʏyƂƂ,ʉz//
	vec3f cx;
	vec3f cy;
	vec3f cz = *target - *origin;	//ʉzɂƂ//
	cz.Normalize();
	cy = (*up) - cz * ((*up) * cz);	//ʏyɂƂ//
	cy.Normalize();
	cx = Cross(cz, cy);				//ʉExɂƂ//
	////////////////////////////////////////////////////////////////////////

	//eƂ̓ςӖs//
	pViewMatrix->Set( cx.x, cy.x, cz.x, 0.0f,
			  cx.y, cy.y, cz.y, 0.0f,
			  cx.z, cy.z, cz.z, 0.0f,
			  -(cx * (*origin)), -(cy * (*origin)), -(cz * (*origin)), 1.0f);

}

void SetProjectionPerspectiveMatrix(float angle, float wph, float z_near, float z_far, mat44f* pProjectionMatrix){
	pProjectionMatrix->Clear();

	pProjectionMatrix->m11 = 1.0f / tan(angle);
	pProjectionMatrix->m22 = pProjectionMatrix->m11 * wph;
	pProjectionMatrix->m33 = z_far / (z_far - z_near);
	pProjectionMatrix->m43 = - z_near * pProjectionMatrix->m33;
	pProjectionMatrix->m34 = 1.0f;
	pProjectionMatrix->m44 = 0.0f;

	//change to coordinate DirectX to OpenGL//
	//pProjectionMatrix->m11 = - pProjectionMatrix->m11;
	/*
	pProjectionMatrix->m13 = - pProjectionMatrix->m13;
	pProjectionMatrix->m23 = - pProjectionMatrix->m23;
	pProjectionMatrix->m33 = - pProjectionMatrix->m33;
	pProjectionMatrix->m43 = - pProjectionMatrix->m43;
	*/
}



//GLŕ`悵erbg}bv摜Ƃĕۑ//
void OutputDX11(VP_FileList& files, const TCHAR *fName, int width, int height, int magnify, CameraGL* cam){
	
	DX11_CORE* dx11_core = s_dx11_core;

	FILE* fp = _tfopen(fName,_T("wb"));
	if (!fp){
		printf("bmpt@C̍쐬Ɏs܂\n");
		return;
	}
	

	
	//(1)DirectX11Ń_O֐.tbv͂Ȃ//
	RenderingDX11(files, width, height, cam);

	
    //(2)XN[tH[}bg擾//
    ID3D11Texture2D* pBackBuffer = NULL;
    HRESULT hr = dx11_core->pSwapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), ( LPVOID* )&pBackBuffer );
    if( FAILED( hr ) ) return;
	
	D3D11_TEXTURE2D_DESC descBackBuffer;
	pBackBuffer->GetDesc(&descBackBuffer);
	pBackBuffer->Release();

	
	//(3)CPUǂݏo\ȃobt@GPUɍ쐬//
    D3D11_TEXTURE2D_DESC Texture2DDesc;
    Texture2DDesc.ArraySize = 1;
    Texture2DDesc.BindFlags = 0;
    Texture2DDesc.CPUAccessFlags = D3D11_CPU_ACCESS_READ;
    Texture2DDesc.Format = descBackBuffer.Format;
    Texture2DDesc.Height = descBackBuffer.Height;
    Texture2DDesc.Width = descBackBuffer.Width;
    Texture2DDesc.MipLevels = 1;
    Texture2DDesc.MiscFlags = 0;
    Texture2DDesc.SampleDesc.Count = 1;
    Texture2DDesc.SampleDesc.Quality = 0;
    Texture2DDesc.Usage = D3D11_USAGE_STAGING;

    ID3D11Texture2D *hCaptureTexture;
    dx11_core->pd3dDevice->CreateTexture2D( &Texture2DDesc, 0, &hCaptureTexture );

	
	//(4)쐬CPUǂݍ݉\obt@GPUŃf[^Rs[//
    ID3D11Resource *hResource;
    dx11_core->pRenderTargetView->GetResource( &hResource );
    dx11_core->pd3dDC->CopyResource( hCaptureTexture, hResource );
	hResource->Release();

	
	//(5)CPŨɃobt@m//
	int size_Color = 4;	
	int line_size = width * size_Color;
	if (line_size % 4){	line_size += 4 - (line_size % 4);}
	BYTE *bmBuf = new BYTE[line_size * height];

	int all_line_size = width * size_Color * magnify;
	if (all_line_size % 4){	all_line_size += 4 - (all_line_size % 4);}
	BYTE *allBuf = new BYTE[all_line_size * height * magnify];
	size_t all_buf_size = all_line_size * height * magnify;

    
	
	//(6)GPU̓ǂݍ݉\obt@CPŨobt@֓]//
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	dx11_core->pd3dDC->Map( hCaptureTexture, 0, D3D11_MAP_READ, 0, &mappedResource);
	CopyMemory(allBuf, mappedResource.pData, all_buf_size);
	dx11_core->pd3dDC->Unmap( hCaptureTexture, 0);
	hCaptureTexture->Release();
    
	
	//(7)CPŨobt@̉摜t@Co//
	BITMAPINFOHEADER bmih;
	BYTE* dib_buf = CreateDIBfromRGBA(allBuf, width * magnify, height * magnify, &bmih);
	writeDIB(fp, &bmih, dib_buf);
	
	delete [] dib_buf;
	delete [] allBuf;
	fclose(fp);



}


/*
//GLŕ`悵erbg}bv摜Ƃĕۑ//
void OutputDX11(){
	
	DX11_CORE* dx11_core = s_dx11_core;



	//(1)DirectX11Ń_O֐.tbv͂Ȃ//
	RenderingDX11();

	
    //(2)obNobt@̃tH[}bg擾//
    ID3D11Texture2D* pBackBuffer = NULL;
    HRESULT hr = dx11_core->pSwapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), ( LPVOID* )&pBackBuffer );
    if( FAILED( hr ) ) return;
	
	D3D11_TEXTURE2D_DESC descBackBuffer;
	pBackBuffer->GetDesc(&descBackBuffer);
	pBackBuffer->Release();


	//(3)CPUǂݏo\ȃobt@GPUɍ쐬//
    D3D11_TEXTURE2D_DESC Texture2DDesc;
    Texture2DDesc.ArraySize = 1;
    Texture2DDesc.BindFlags = 0;
    Texture2DDesc.CPUAccessFlags = D3D11_CPU_ACCESS_READ;
    Texture2DDesc.Format = descBackBuffer.Format;
    Texture2DDesc.Height = descBackBuffer.Height;
    Texture2DDesc.Width = descBackBuffer.Width;
    Texture2DDesc.MipLevels = 1;
    Texture2DDesc.MiscFlags = 0;
    Texture2DDesc.SampleDesc.Count = 1;
    Texture2DDesc.SampleDesc.Quality = 0;
    Texture2DDesc.Usage = D3D11_USAGE_STAGING;

    ID3D11Texture2D *hCaptureTexture;
    dx11_core->pd3dDevice->CreateTexture2D( &Texture2DDesc, 0, &hCaptureTexture );

	
	//(4)쐬CPUǂݍ݉\obt@GPUŃf[^Rs[//
    ID3D11Resource *hResource;
    dx11_core->pRenderTargetView->GetResource( &hResource );
    dx11_core->pd3dDC->CopyResource( hCaptureTexture, hResource );
	hResource->Release();

	
	//(5)CPŨɃobt@m//
	double width = descBackBuffer.Width;
	double height = descBackBuffer.Height;
	size_t buffer_size = width * height * 4;
	BYTE *bmp_buffer = new BYTE[buffer_size];


	//(6)GPU̓ǂݍ݉\obt@CPŨobt@֓]//
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	dx11_core->pd3dDC->Map( hCaptureTexture, 0, D3D11_MAP_READ, 0, &mappedResource);
	CopyMemory(bmp_buffer, mappedResource.pData, buffer_size);
	dx11_core->pd3dDC->Unmap( hCaptureTexture, 0);
	hCaptureTexture->Release();
    
	//(7)CPŨobt@̉摜t@Co//
	SaveBMP(bmp_buffer, width, height);
	
	delete [] bmp_buffer;


}
*/