

#include "ResetBond.h"

#include "LinkedCellList.h"



extern int g_periodic;

template <class TBVECTOR>
class NBOND_WORKER_ADD_BOND{
private:
	int m_bond_index;
	TBVECTOR* m_bvector;
	BVEC_REGION* m_bvecStartEnd;
	
	int m_start_point;
	int m_twobodyendpoint;

public:
	NBOND_WORKER_ADD_BOND(TBVECTOR* bvector, BVEC_REGION* bvecStartEnd, int bond_index) : 
		m_bvector(bvector), m_bvecStartEnd(bvecStartEnd), m_bond_index(bond_index) 
	{
		
	};

	~NBOND_WORKER_ADD_BOND(){};

	void Begin(int i){
		//m_bond_index = m_bond_par_atom * i;
		m_start_point = m_bond_index;

	};

	void Work(int i, int j, const vec3d& dd, const double r2) {
			// add to bonding array //
		TBVECTOR& bv_i = m_bvector[m_bond_index];

		bv_i.i = i;
		bv_i.j = j;
		bv_i.v.x = -(float)(dd.x);
		bv_i.v.y = -(float)(dd.y);
		bv_i.v.z = -(float)(dd.z);
				
		// increment of index //
		m_bond_index ++;
	};

	void Intermidiate(){
		m_twobodyendpoint = m_bond_index;	//end point for two-body force of i-th particle's bond//
	}

	void End(int i){

		m_bvecStartEnd[i].twobodyendpoint = m_twobodyendpoint;
		m_bvecStartEnd[i].startpoint = m_start_point;	//start point of i-th particle's bond array//
		m_bvecStartEnd[i].endpoint = m_bond_index;	//end point of i-th particle's bond//

	};

};


void glips_ResetBond(ATOMS_DATA *dat, BOND_INFO* bond, double cutoff){
	//{hČvZ


	mat33f boxaxis = dat->boxaxis;
	vec3f boxorg = dat->boxorg;
		

	if (g_periodic == 0){
		
		//Ẽ{bNX̒`//
		vec3f rmin = {0.0f, 0.0f, 0.0f};
		vec3f rmax = {0.0f, 0.0f, 0.0f};
		for(int i = 0; i < dat->pcnt; i++){
			if( rmin.x  > dat->r[i].x  ){ rmin.x = dat->r[i].x;}
			if( rmin.y  > dat->r[i].y ){ rmin.y = dat->r[i].y;}
			if( rmin.z  > dat->r[i].z ){ rmin.z = dat->r[i].z;}

			if( rmax.x  < dat->r[i].x ){ rmax.x = dat->r[i].x;}
			if( rmax.y  < dat->r[i].y ){ rmax.y = dat->r[i].y;}
			if( rmax.z  < dat->r[i].z ){ rmax.z = dat->r[i].z;}
		}
		/*
		rmin.x -= (float)BOND_LENGTH;
		rmin.y -= (float)BOND_LENGTH;
		rmin.z -= (float)BOND_LENGTH;
		rmax.x += (float)BOND_LENGTH;
		rmax.y += (float)BOND_LENGTH;
		rmax.z += (float)BOND_LENGTH;
		*/

		boxorg.Set(rmin.x, rmin.y, rmin.z);
		boxaxis.Set((rmax.x - rmin.x), 0.0f, 0.0f, 0.0f, (rmax.y - rmin.y),0.0f, 0.0f, 0.0f, (rmax.z - rmin.z));

		

		
	//	dat->boxaxis = boxaxis;
	//	dat->boxorg = boxorg;
	}




////ԑ===============================
//LARGE_INTEGER start_pc,end_pc,freq_pc;
//double sec_pc[10];
//QueryPerformanceFrequency(&freq_pc);
//QueryPerformanceCounter(&start_pc);
////ԑ===============================

	vec3d* r_d = new vec3d[dat->pcnt];
	
	for(int i=0;i<dat->pcnt;i++){
		r_d[i] = dat->r[i] - boxorg;
	}

	LCL* pLCL = new LCL( cutoff );

	pLCL->SetMaxNumParticles(dat->pcnt);
	
	
	
	double nonortho_term = fabs(boxaxis.m12) + fabs(boxaxis.m13)
					+ fabs(boxaxis.m21) + fabs(boxaxis.m23) 
					+ fabs(boxaxis.m31) + fabs(boxaxis.m32); 
	if(nonortho_term > 0.1e-6){
		mat33d Am = boxaxis;
		pLCL->SetBoxAxis(Am); 
	}else{
		pLCL->SetBoxsize(boxaxis.m11, boxaxis.m22, boxaxis.m33);
	}
	
	pLCL->Create(dat->pcnt, r_d, NULL);
	
	
	
	int bond_buf_sz = dat->pcnt * 12;

	if(bond->sz_bvector < bond_buf_sz){
		bond->sz_bvector  = bond_buf_sz;
		delete [] (bond->bvector );
		bond->bvector = new VECTOR_IJ[(bond->sz_bvector)];
	}


	BVEC_REGION* bvStartEnd = new BVEC_REGION[dat->pcnt];
	
			
	
	LoopBondList(pLCL, 0, dat->pcnt, r_d, cutoff*cutoff, NBOND_WORKER_ADD_BOND<VECTOR_IJ>(bond->bvector, bvStartEnd, 0));

	
	if(bond->sz_coords < dat->pcnt){
		bond->sz_coords = dat->pcnt;
		delete [] (bond->coords);
		bond->coords = new int[(bond->sz_coords)];
	}
	int* pn = bond->coords;

	int bondcnt = 0;
	for(int i = 0 ; i < dat->pcnt; i++){
		pn[i] = bvStartEnd[i].endpoint - bvStartEnd[i].startpoint;
		bondcnt += pn[i];
			
#ifndef KIND_He
#define KIND_He   (2)
#endif
		if(dat->knd[i] == KIND_He){
			*pn = 0;
		}
	
		
	}
	bond->count = bondcnt;


	

	if(g_periodic){
		for(int i=0;i<dat->pcnt;i++){
			dat->r[i] = (vec3f)(r_d[i]) + boxorg;
		}
	}

	delete r_d;
	delete bvStartEnd;
	delete pLCL;

}
