
////////////////////////////////////////////////////////
//
//  AFS potential//
//
/////////////////////////////////////////////////////////


#ifndef NBonds_H
#define NBonds_H

#pragma once



#ifdef _OPENMP
#include <omp.h>
#endif	//_OPENMP
#include "myomptune.h"


#include "vec3.h"
#include "mat33.h"
#include "IPotential.h"
#include "LinkedCellList.h"
#include "atomarray.h"
//#include "debugprint.h"

	
class NBonds : public IPotential {
private:

	LCL* m_pLCL;

	static const int MAX_NUM_BOND = 24;		// the avarage of bond number for each atom.
		
	struct BVECTOR{
		int pairIndex;	// index j which is pair of i //
		vec3d rij;		// normalized relative vector //
		double dr;
	};

	struct BVEC_REGION{
		int startpoint;		// start index of i-th atom's bond in BOP_BVECTOR array //
		int twobodyendpoint;
		int endpoint;		// end index of i-th atom's bond in BOP_BVECTOR array //
	};

	BVECTOR* m_bvector;	//omp:shared//
	BVEC_REGION* m_bvecStartEnd;
	//int m_ijmax;
	
	double* m_Jij;
	mat33d m_Fr_m; //F x r		//omp:gatherd//

	
	void AllocateBuffer(int maxcount);	

	void Reset(int count);
	void BondListDualDirectS(int count);
	void ForceByBondList(int count, vec3d* __restrict a_force, vec3d* __restrict sh_force, double* __restrict a_pot, mat33d* __restrict a_Fr_m);

	void AddFr(vec3d frc, vec3d dr, mat33d* a_Fr_m);
	
//	static const int MAX_NUM_BOND;
//	double m_CUTOFF_LENGTH;
//	double m_CUTOFF_LENGTH2;

	int* m_knd;			//omp:shared//
	vec3d* m_r;			//omp:shared//
	vec3d* m_p;			//omp:shared//
	vec3d* m_f;			//omp:shared|gathered//
	double* m_mass;		//omp:shared//
	double* m_pot;		//omp:shared|gathered//
	
	vec3d* sh_aforce;	//omp:gatherd//
//	double* sh_apot;	//omp:gatherd//

	// NBondsŗL̕ϐ ////////////////////////////////////////////////////
	int* m_num_bonds;	//ӓdqx.

	double m_cutoff;	//cutoff length.
	double m_cutoff2;	//square of cutoff length.

public:


	NBonds(int maxcount);
	virtual ~NBonds();
	
	void SetMaxNumParticles(int maxcount){ 
		AllocateBuffer(maxcount);
		m_pLCL->SetMaxNumParticles(maxcount);
	};

	void SetParticlePointers(AtomArray* atoms);
	void SetLinkedCellList(LCL* pLCL){m_pLCL = pLCL;};
	void SetBoxsize(double x, double y, double z){m_pLCL->SetBoxsize(x, y, z);};
	void SetBoxAxis(const vec3d& a, const vec3d& b, const vec3d& c){m_pLCL->SetBoxAxis(a, b, c);};
	void SetBoxAxis(const mat33d& Am){m_pLCL->SetBoxAxis(Am);};

	void Force(int count, int noloop_count);

	void GetFr(mat33d* a_Fr_m);
	
	double GetCutoffLength();
	double GetForceRange();
	
	void GetParams(double* params){};
	void SetParams(const double* params);
	int GetNumParams(){return 0;};	//DownfoldingŒłp[^[̐Ԃ.
	
	void GetInfo(int mode, void* params);
	
};




#endif	//!NBonds_H

