
#ifndef __mat33_h__
#define __mat33_h__


#pragma once

#include "vec3.h"

#include <math.h>
#include <string.h>



	struct mat33f;

//double================================
	struct mat33d{
	public:
		union {
			double elements[9];
			struct {		//̂Ƃɒ(c)
				double m11;
				double m21;
				double m31;
				double m12;
				double m22;
				double m32;
				double m13;
				double m23;
				double m33;
			};
			struct {
				vec3d a;
				vec3d b;
				vec3d c;
			};
		};

		inline mat33d& Set(	const double& a11, const double& a12, const double& a13,
							const double& a21, const double& a22, const double& a23,
							const double& a31, const double& a32, const double& a33){
			m11 = a11;
			m21 = a21;
			m31 = a31;
			m12 = a12;
			m22 = a22;
			m32 = a32;
			m13 = a13;
			m23 = a23;
			m33 = a33;
			return *this;
		};


		inline mat33d& operator +=(const mat33d& m){
			a += m.a;
			b += m.b;
			c += m.c;
			return *this;
		}
		
		inline mat33d operator -(){
			mat33d m2;
			m2.a = - a;
			m2.b = - b;
			m2.c = - c;
			return m2;
		}

		inline mat33d& operator -=(const mat33d& m){
			a -= m.a;
			b -= m.b;
			c -= m.c;
			return *this;
		}

		inline mat33d& operator *=(const double& d){
			m11 *= d;
			m21 *= d;
			m31 *= d;
			m12 *= d;
			m22 *= d;
			m32 *= d;
			m13 *= d;
			m23 *= d;
			m33 *= d;
			return *this;
		};

		inline mat33d& operator /=(const double& d){
			double di = 1.0 / d;
			m11 *= di;
			m21 *= di;
			m31 *= di;
			m12 *= di;
			m22 *= di;
			m32 *= di;
			m13 *= di;
			m23 *= di;
			m33 *= di;
			return *this;
		};
		
		//]u
		inline void SetTranspose(){
			double temp;
			temp = m12;
			m12 = m21;
			m21 = temp;
			
			temp = m13;
			m13 = m31;
			m31 = temp;
			
			temp = m23;
			m23 = m32;
			m32 = temp;
			
		}
			
		inline void Clear(){
			memset(elements, 0, 9 * sizeof(double));
		}
	
		inline operator mat33f();

	};

	inline mat33d operator +(const mat33d& m, const mat33d& m2){
		mat33d mr;
		mr.a = m.a + m2.a;
		mr.b = m.b + m2.b;
		mr.c = m.c + m2.c;
		return mr;
	}

	inline mat33d operator -(const mat33d& m, const mat33d& m2){
		mat33d mr;
		mr.a = m.a - m2.a;
		mr.b = m.b - m2.b;
		mr.c = m.c - m2.c;
		return mr;
	}
	
	inline mat33d operator *(const mat33d& m, const mat33d& m2){
		mat33d mr;
		mr.m11 = m.m11 * m2.m11 + m.m12 * m2.m21 + m.m13 * m2.m31;
		mr.m12 = m.m11 * m2.m12 + m.m12 * m2.m22 + m.m13 * m2.m32;
		mr.m13 = m.m11 * m2.m13 + m.m12 * m2.m23 + m.m13 * m2.m33;
		mr.m21 = m.m21 * m2.m11 + m.m22 * m2.m21 + m.m23 * m2.m31;
		mr.m22 = m.m21 * m2.m12 + m.m22 * m2.m22 + m.m23 * m2.m32;
		mr.m23 = m.m21 * m2.m13 + m.m22 * m2.m23 + m.m23 * m2.m33;
		mr.m31 = m.m31 * m2.m11 + m.m32 * m2.m21 + m.m33 * m2.m31;
		mr.m32 = m.m31 * m2.m12 + m.m32 * m2.m22 + m.m33 * m2.m32;
		mr.m33 = m.m31 * m2.m13 + m.m32 * m2.m23 + m.m33 * m2.m33;
		return mr;
	}
	
	inline vec3d operator *(const mat33d& m, const vec3d& v){
		vec3d vr;
		vr = m.a * v.x;
		vr += m.b * v.y;
		vr += m.c * v.z;
		return vr;
	}
	
	inline mat33d operator *(const mat33d& m, const double& d){
		mat33d m2;
		m2.a = m.a * d;
		m2.b = m.b * d;
		m2.c = m.c * d;
		return m2;
	}

	inline mat33d operator /(const mat33d& m, const double& d){
		double di = 1.0 / d;
		mat33d m2;
		m2.m11 = m.m11 * di;
		m2.m21 = m.m21 * di;
		m2.m31 = m.m31 * di;
		m2.m12 = m.m12 * di;
		m2.m22 = m.m22 * di;
		m2.m32 = m.m32 * di;
		m2.m13 = m.m13 * di;
		m2.m23 = m.m23 * di;
		m2.m33 = m.m33 * di;
		return m2;
	}

	
	inline double Det(mat33d& m){
		return m.m11 * (m.m22 * m.m33 - m.m23* m.m32) + m.m12 * (m.m23 * m.m31 - m.m21 * m.m33) + m.m13 * (m.m21 * m.m32 - m.m22 * m.m31);
	}

	inline mat33d Inverse(mat33d& m){
		mat33d m2;
		m2.m11 = m.b.y*m.c.z-m.b.z*m.c.y;
		m2.m12 = m.b.z*m.c.x-m.b.x*m.c.z;
		m2.m13 = m.b.x*m.c.y-m.b.y*m.c.x;
		m2.m21 = m.c.y*m.a.z-m.c.z*m.a.y;
		m2.m22 = m.c.z*m.a.x-m.c.x*m.a.z;
		m2.m23 = m.c.x*m.a.y-m.c.y*m.a.x;
		m2.m31 = m.a.y*m.b.z-m.a.z*m.b.y;
		m2.m32 = m.a.z*m.b.x-m.a.x*m.b.z;
		m2.m33 = m.a.x*m.b.y-m.a.y*m.b.x;
		m2 = m2 / (Det(m));
		return m2;
	}	
	
	inline mat33d Transpose(mat33d& m){
		mat33d m2;
		m2.m11 = m.m11;
		m2.m21 = m.m12;
		m2.m31 = m.m13;
		m2.m12 = m.m21;
		m2.m22 = m.m22;
		m2.m32 = m.m23;
		m2.m13 = m.m31;
		m2.m23 = m.m32;
		m2.m33 = m.m33;
		return m2;
	}
	
	inline double Trace(mat33d& m){
		return m.m11 + m.m22 + m.m33;
	}
		
//float================================
	struct mat33f{
	public:
		union {
			float elements[9];
			struct {		//̂Ƃɒ(c)
				float m11;
				float m21;
				float m31;
				float m12;
				float m22;
				float m32;
				float m13;
				float m23;
				float m33;
			};
			struct {
				vec3f a;
				vec3f b;
				vec3f c;
			};
		};

		inline mat33f& Set(	const float& a11, const float& a12, const float& a13,
							const float& a21, const float& a22, const float& a23,
							const float& a31, const float& a32, const float& a33){
			m11 = a11;
			m21 = a21;
			m31 = a31;
			m12 = a12;
			m22 = a22;
			m32 = a32;
			m13 = a13;
			m23 = a23;
			m33 = a33;
			return *this;
		};


		inline mat33f& operator +=(const mat33f& m){
			a += m.a;
			b += m.b;
			c += m.c;
			return *this;
		}
		
		inline mat33f operator -(){
			mat33f m2;
			m2.a = - a;
			m2.b = - b;
			m2.c = - c;
			return m2;
		}

		inline mat33f& operator -=(const mat33f& m){
			a -= m.a;
			b -= m.b;
			c -= m.c;
			return *this;
		}
		
		inline mat33f& operator *=(const float& d){
			m11 *= d;
			m21 *= d;
			m31 *= d;
			m12 *= d;
			m22 *= d;
			m32 *= d;
			m13 *= d;
			m23 *= d;
			m33 *= d;
			return *this;
		};

		inline mat33f& operator /=(const float& d){
			float di = 1.0f / d;
			m11 *= di;
			m21 *= di;
			m31 *= di;
			m12 *= di;
			m22 *= di;
			m32 *= di;
			m13 *= di;
			m23 *= di;
			m33 *= di;
			return *this;
		};
		
		//]u
		inline void SetTranspose(){
			float temp;
			temp = m12;
			m12 = m21;
			m21 = temp;
			
			temp = m13;
			m13 = m31;
			m31 = temp;
			
			temp = m23;
			m23 = m32;
			m32 = temp;
			
		}
			
		inline void Clear(){
			memset(elements, 0, 9 * sizeof(float));
		}
	

		inline operator mat33d();

	};

	inline mat33f operator +(const mat33f& m, const mat33f& m2){
		mat33f mr;
		mr.a = m.a + m2.a;
		mr.b = m.b + m2.b;
		mr.c = m.c + m2.c;
		return mr;
	}

	inline mat33f operator -(const mat33f& m, const mat33f& m2){
		mat33f mr;
		mr.a = m.a - m2.a;
		mr.b = m.b - m2.b;
		mr.c = m.c - m2.c;
		return mr;
	}
	
	inline mat33f operator *(const mat33f& m, const mat33f& m2){
		mat33f mr;
		mr.m11 = m.m11 * m2.m11 + m.m12 * m2.m21 + m.m13 * m2.m31;
		mr.m12 = m.m11 * m2.m12 + m.m12 * m2.m22 + m.m13 * m2.m32;
		mr.m13 = m.m11 * m2.m13 + m.m12 * m2.m23 + m.m13 * m2.m33;
		mr.m21 = m.m21 * m2.m11 + m.m22 * m2.m21 + m.m23 * m2.m31;
		mr.m22 = m.m21 * m2.m12 + m.m22 * m2.m22 + m.m23 * m2.m32;
		mr.m23 = m.m21 * m2.m13 + m.m22 * m2.m23 + m.m23 * m2.m33;
		mr.m31 = m.m31 * m2.m11 + m.m32 * m2.m21 + m.m33 * m2.m31;
		mr.m32 = m.m31 * m2.m12 + m.m32 * m2.m22 + m.m33 * m2.m32;
		mr.m33 = m.m31 * m2.m13 + m.m32 * m2.m23 + m.m33 * m2.m33;
		return mr;
	}
	
	inline vec3f operator *(const mat33f& m, const vec3f& v){
		vec3f vr;
		vr = m.a * v.x;
		vr += m.b * v.y;
		vr += m.c * v.z;
		return vr;
	}
	
	inline mat33f operator *(const mat33f& m, const float& d){
		mat33f m2;
		m2.a = m.a * d;
		m2.b = m.b * d;
		m2.c = m.c * d;
		return m2;
	}

	inline mat33f operator /(const mat33f& m, const float& d){
		float di = 1.0f / d;
		mat33f m2;
		m2.m11 = m.m11 * di;
		m2.m21 = m.m21 * di;
		m2.m31 = m.m31 * di;
		m2.m12 = m.m12 * di;
		m2.m22 = m.m22 * di;
		m2.m32 = m.m32 * di;
		m2.m13 = m.m13 * di;
		m2.m23 = m.m23 * di;
		m2.m33 = m.m33 * di;
		return m2;
	}

	
	inline float Det(mat33f& m){
		return m.m11 * (m.m22 * m.m33 - m.m23* m.m32) + m.m12 * (m.m23 * m.m31 - m.m21 * m.m33) + m.m13 * (m.m21 * m.m32 - m.m22 * m.m31);
	}

	inline mat33f Inverse(mat33f& m){
		mat33f m2;
		m2.m11 = m.b.y*m.c.z-m.b.z*m.c.y;
		m2.m12 = m.b.z*m.c.x-m.b.x*m.c.z;
		m2.m13 = m.b.x*m.c.y-m.b.y*m.c.x;
		m2.m21 = m.c.y*m.a.z-m.c.z*m.a.y;
		m2.m22 = m.c.z*m.a.x-m.c.x*m.a.z;
		m2.m23 = m.c.x*m.a.y-m.c.y*m.a.x;
		m2.m31 = m.a.y*m.b.z-m.a.z*m.b.y;
		m2.m32 = m.a.z*m.b.x-m.a.x*m.b.z;
		m2.m33 = m.a.x*m.b.y-m.a.y*m.b.x;
		m2 = m2 / (Det(m));
		return m2;
	}
	
	
	inline mat33f Transpose(mat33f& m){
		mat33f m2;
		m2.m11 = m.m11;
		m2.m21 = m.m12;
		m2.m31 = m.m13;
		m2.m12 = m.m21;
		m2.m22 = m.m22;
		m2.m32 = m.m23;
		m2.m13 = m.m31;
		m2.m23 = m.m32;
		m2.m33 = m.m33;
		return m2;
	}
	
	inline float Trace(mat33f& m){
		return m.m11 + m.m22 + m.m33;
	}
		
//cast================================
	inline mat33d::operator mat33f(){
		mat33f mf;
		mf.m11 = (float)m11;
		mf.m21 = (float)m21;
		mf.m31 = (float)m31;
		mf.m12 = (float)m12;
		mf.m22 = (float)m22;
		mf.m32 = (float)m32;
		mf.m13 = (float)m13;
		mf.m23 = (float)m23;
		mf.m33 = (float)m33;
		return mf;
	}

	inline mat33f::operator mat33d(){
		mat33d md;
		md.m11 = (double)m11;
		md.m21 = (double)m21;
		md.m31 = (double)m31;
		md.m12 = (double)m12;
		md.m22 = (double)m22;
		md.m32 = (double)m32;
		md.m13 = (double)m13;
		md.m23 = (double)m23;
		md.m33 = (double)m33;
		return md;
	}


#endif // __mat33_h__