
#ifndef __vec3_h__
#define __vec3_h__


#pragma once

#include <math.h>
#include <string.h>

	struct vec3f;

//double================================
	struct vec3d{
	public:
		double x;
		double y;
		double z;

		inline vec3d& Set(const double& d1, const double& d2, const double& d3){
			x = d1;
			y = d2;
			z = d3;
			return *this;
		};

		inline vec3d& operator +=(const vec3d& v){
			x += v.x;
			y += v.y;
			z += v.z;
			return *this;
		}
		
		inline vec3d operator -(){
			vec3d v2;
			v2.x = - x;
			v2.y = - y;
			v2.z = - z;
			return v2;
		}

		inline vec3d& operator -=(const vec3d& v){
			x -= v.x;
			y -= v.y;
			z -= v.z;
			return *this;
		}
		
		inline vec3d& operator *=(const double& d){
			x *= d;
			y *= d;
			z *= d;
			return *this;
		}

		inline vec3d& operator /=(const double& d){
			double di = 1.0 / d;
			x *= di;
			y *= di;
			z *= di;
			return *this;
		}
		
		inline void Clear(){
			memset(&x, 0, 3 * sizeof(double));
		}
		
		inline operator vec3f();

		
		inline void Normalize(){
			double ni = 1.0/ sqrt(x*x + y*y + z*z);
			*this *= ni;
		}
	
		inline vec3d GetUnit();
	
	};

	inline double operator *(const vec3d& v, const vec3d& v2){
		return (v.x * v2.x + v.y * v2.y + v.z * v2.z);
	}
	
	inline vec3d operator +(const vec3d& v, const vec3d& v2){
		vec3d vr;
		vr.x = v.x + v2.x;
		vr.y = v.y + v2.y;
		vr.z = v.z + v2.z;
		return vr;
	}

	inline vec3d operator -(const vec3d& v, const vec3d& v2){
	//inline vec3d operator -(const vec3d v, const vec3d v2){
		vec3d vr;
		vr.x = v.x - v2.x;
		vr.y = v.y - v2.y;
		vr.z = v.z - v2.z;
		return vr;
	}

	inline vec3d operator *(const vec3d& v, const double& d){
		vec3d v2;
		v2.x = v.x * d;
		v2.y = v.y * d;
		v2.z = v.z * d;
		return v2;
	}
	
	inline vec3d operator /(const vec3d& v, const double& d){
		double di = 1.0 / d;
		vec3d v2;
		v2.x = v.x * di;
		v2.y = v.y * di;
		v2.z = v.z * di;
		return v2;
	}			

	
	inline vec3d Unit(const vec3d& v){
		double ni = 1.0/ sqrt(v*v);
		vec3d vr = v * ni;
		return vr;			
	}

	inline vec3d Cross(const vec3d& v, const vec3d& v2){
		vec3d v3;
		v3.x = v.y * v2.z - v.z * v2.y;
		v3.y = v.z * v2.x - v.x * v2.z;
		v3.z = v.x * v2.y - v.y * v2.x;
		return v3;
	}

	inline double Abs(const vec3d& v){
		return sqrt(v*v);
	}



//float================================
	struct vec3f{
	public:
		float x;
		float y;
		float z;

		inline vec3f& Set(const float& f1, const float& f2, const float& f3){
			x = f1;
			y = f2;
			z = f3;
			return *this;
		};

		inline vec3f& operator +=(const vec3f& v){
			x += v.x;
			y += v.y;
			z += v.z;
			return *this;
		}

		inline vec3f operator -(){
			vec3f v2;
			v2.x = - x;
			v2.y = - y;
			v2.z = - z;
			return v2;
		}

		inline vec3f& operator -=(const vec3f& v){
			x -= v.x;
			y -= v.y;
			z -= v.z;
			return *this;
		}
		
		inline vec3f& operator *=(const float& f){
			x *= f;
			y *= f;
			z *= f;
			return *this;
		}
	
		inline vec3f& operator /=(const float& f){
			float fi = 1.0f / f;
			x *= fi;
			y *= fi;
			z *= fi;
			return *this;
		}

		inline void Clear(){
			memset(&x, 0, 3 * sizeof(float));
		}

		inline void Normalize(){
			float ni = (float)(1.0/ sqrt(x*x + y*y + z*z));
			*this *= ni;
		}
	
		inline operator vec3d();

	};

	inline vec3f operator *(const vec3f& v, const float& f){
		vec3f v2;
		v2.x = v.x * f;
		v2.y = v.y * f;
		v2.z = v.z * f;
		return v2;
	}
	
	inline vec3f operator /(const vec3f& v, const float& f){
		float fi = 1.0f / f;
		vec3f v2;
		v2.x = v.x * fi;
		v2.y = v.y * fi;
		v2.z = v.z * fi;
		return v2;
	}			
	
	inline float operator *(const vec3f& v, const vec3f& v2){
		return (v.x * v2.x + v.y * v2.y + v.z * v2.z);
	}
	
	inline vec3f operator +(const vec3f& v, const vec3f& v2){
		vec3f vr;
		vr.x = v.x + v2.x;
		vr.y = v.y + v2.y;
		vr.z = v.z + v2.z;
		return vr;
	}

	inline vec3f operator -(const vec3f& v, const vec3f& v2){
		vec3f vr;
		vr.x = v.x - v2.x;
		vr.y = v.y - v2.y;
		vr.z = v.z - v2.z;
		return vr;
	}

	inline vec3f GetUnit(const vec3f& v){
		float ni = (float)(1.0/ sqrt((double)(v*v)));
		vec3f vr = v * ni;
		return vr;			
	}

	inline vec3f Cross(const vec3f &v, const vec3f &v2){
		vec3f v3;
		v3.x = v.y * v2.z - v.z * v2.y;
		v3.y = v.z * v2.x - v.x * v2.z;
		v3.z = v.x * v2.y - v.y * v2.x;
		return v3;
	}
		
	inline float Abs(const vec3f& v){
		return (float)sqrt(v*v);
	}

	
	struct vec3i{
	public:
		
		int x;
		int y;
		int z;

		vec3i(){
			x = 1;
			y = 2;
			z = 3;
		};
	};

//cast================================
	inline vec3d::operator vec3f(){
		vec3f vf;
		vf.x = (float)x;
		vf.y = (float)y;
		vf.z = (float)z;
		return vf;
	}
	
	inline vec3f::operator vec3d(){
		vec3d vd;
		vd.x = (double)x;
		vd.y = (double)y;
		vd.z = (double)z;
		return vd;
	}
	

#endif // __vec3_h__